@@ -1,33 +1,44 @@
 /*
  * pg_dbms_stats.c
  *
- * Copyright (c) 2009-2014, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
+ * Copyright (c) 2009-2017, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
  * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  */
 #include "postgres.h"
 
+#include "access/sysattr.h"
 #include "access/transam.h"
+#include "catalog/pg_index.h"
 #include "catalog/pg_statistic.h"
 #include "catalog/pg_type.h"
+#include "catalog/namespace.h"
+#include "catalog/pg_authid.h"
 #include "commands/trigger.h"
 #include "executor/spi.h"
 #include "funcapi.h"
 #include "optimizer/plancat.h"
+#include "parser/parse_oper.h"
+#include "parser/parsetree.h"
 #include "storage/bufmgr.h"
+#include "utils/acl.h"
 #include "utils/builtins.h"
+#include "utils/elog.h"
+#include "utils/fmgroids.h"
 #include "utils/guc.h"
 #include "utils/inval.h"
 #include "utils/lsyscache.h"
 #include "utils/selfuncs.h"
 #include "utils/syscache.h"
-#if PG_VERSION_NUM >= 90200
+#include "miscadmin.h"
 #include "utils/rel.h"
-#endif
 #if PG_VERSION_NUM >= 90300
 #include "access/htup_details.h"
 #include "utils/catcache.h"
 #endif
+#if PG_VERSION_NUM >= 100000
+#include <math.h>
+#endif
 
 #include "pg_dbms_stats.h"
 
@@ -39,6 +50,24 @@ PG_MODULE_MAGIC;
 
 #define MAX_REL_CACHE		50		/* expected max # of rel stats entries */
 
+/* acl_ok must be set after the fix for CVE-2017-7484 */
+#if PG_VERSION_NUM >= 100000 || \
+	(PG_VERSION_NUM >=  90603 && PG_VERSION_NUM < 100000) ||	\
+	(PG_VERSION_NUM >=  90507 && PG_VERSION_NUM <  90600) ||	\
+	(PG_VERSION_NUM >=  90412 && PG_VERSION_NUM <  90500) ||	\
+	(PG_VERSION_NUM >=  90317 && PG_VERSION_NUM <  90400) ||	\
+	(PG_VERSION_NUM >=  90221 && PG_VERSION_NUM <  90300)
+#define PGDS_HAVE_ACL_OK
+/*
+ * acl_ok of the returning VariableStatData must be set if set_acl_okk is
+ * true. The code is compiled only if the compile target PG version is match
+ * the above conditions. Conversely, acl_ok is added to the end of
+ * VariableStatData so we can safely omit setting it when the PG version
+ * pg_dbms_stats is loaded onto is out of the conditions.
+ */
+static	bool set_acl_ok = false;
+#endif
+
 /* Relation statistics cache entry */
 typedef struct StatsRelationEntry
 {
@@ -65,6 +94,7 @@ typedef struct StatsColumnEntry
 {
   bool		negative;
   int32		attnum;
+  bool		inh;
   HeapTuple tuple;
 } StatsColumnEntry;
 
@@ -74,20 +104,22 @@ get_attavgwidth_hook_type		prev_get_attavgwidth = NULL;
 get_relation_stats_hook_type	prev_get_relation_stats = NULL;
 get_index_stats_hook_type		prev_get_index_stats = NULL;
 
+/* namings */
+#define NSPNAME "dbms_stats"
+#define RELSTAT_TBLNAME "relation_stats_locked"
+#define COLSTAT_TBLNAME "column_stats_locked"
+
 /* rows_query(oid) RETURNS int4, float4, int4 */
 static const char  *rows_query =
-	"SELECT relpages, reltuples, curpages"
-#if PG_VERSION_NUM >= 90200
-	", relallvisible"
-#endif
-	"  FROM dbms_stats.relation_stats_locked"
+	"SELECT relpages, reltuples, curpages, relallvisible"
+	"  FROM " NSPNAME "." RELSTAT_TBLNAME
 	" WHERE relid = $1";
 static SPIPlanPtr	rows_plan = NULL;
 
 /* tuple_query(oid, int2, bool) RETURNS pg_statistic */
 static const char  *tuple_query =
 	"SELECT * "
-	"  FROM dbms_stats.column_stats_locked "
+	"  FROM " NSPNAME "." COLSTAT_TBLNAME
 	" WHERE starelid = $1 "
 	"   AND staattnum = $2 "
 	"   AND stainherit = $3";
@@ -104,6 +136,12 @@ static int			nested_level = 0;
  */
 static HTAB	   *rel_stats;
 
+/*
+ * The owner of pg_dbms_stats statistic tables.
+ */
+static Oid	stats_table_owner = InvalidOid;
+static char *stats_table_owner_name = "";
+
 #define get_pg_statistic(tuple)	((Form_pg_statistic) GETSTRUCT(tuple))
 
 PG_FUNCTION_INFO_V1(dbms_stats_merge);
@@ -111,12 +149,18 @@ PG_FUNCTION_INFO_V1(dbms_stats_invalidate_relation_cache);
 PG_FUNCTION_INFO_V1(dbms_stats_invalidate_column_cache);
 PG_FUNCTION_INFO_V1(dbms_stats_is_system_schema);
 PG_FUNCTION_INFO_V1(dbms_stats_is_system_catalog);
+PG_FUNCTION_INFO_V1(dbms_stats_anyary_anyary);
+PG_FUNCTION_INFO_V1(dbms_stats_type_is_analyzable);
+PG_FUNCTION_INFO_V1(dbms_stats_anyarray_basetype);
 
 extern Datum dbms_stats_merge(PG_FUNCTION_ARGS);
 extern Datum dbms_stats_invalidate_relation_cache(PG_FUNCTION_ARGS);
 extern Datum dbms_stats_invalidate_column_cache(PG_FUNCTION_ARGS);
 extern Datum dbms_stats_is_system_schema(PG_FUNCTION_ARGS);
 extern Datum dbms_stats_is_system_catalog(PG_FUNCTION_ARGS);
+extern Datum dbms_stats_anyary_anyary(PG_FUNCTION_ARGS);
+extern Datum dbms_stats_type_is_analyzable(PG_FUNCTION_ARGS);
+extern Datum dbms_stats_anyarray_basetype(PG_FUNCTION_ARGS);
 
 static HeapTuple dbms_stats_merge_internal(HeapTuple lhs, HeapTuple rhs,
 	TupleDesc tupledesc);
@@ -143,7 +187,8 @@ static HeapTuple get_merged_column_stats(Oid relid, AttrNumber attnum,
 	bool inh);
 static HeapTuple column_cache_search(Oid relid, AttrNumber attnum,
 									 bool inh, bool*negative);
-static HeapTuple column_cache_enter(Oid relid, int32 attnum, HeapTuple tuple);
+static HeapTuple column_cache_enter(Oid relid, int32 attnum, bool inh,
+									HeapTuple tuple);
 static bool execute_plan(SPIPlanPtr *plan, const char *query, Oid relid,
 	const AttrNumber *attnum, bool inh);
 static void StatsCacheRelCallback(Datum arg, Oid relid);
@@ -182,6 +227,25 @@ _PG_init(void)
 	}
 #endif
 
+#ifdef PGDS_HAVE_ACL_OK
+	{
+		/*
+		 * Check the PG version this module loaded onto. This aid is required
+		 * for binary backward compatibility within a major PG version.
+		 */
+		int major_version = PG_VERSION_NUM / 100;
+		int minor_version = PG_VERSION_NUM % 100;
+
+		if (major_version >= 1000 ||
+			(major_version == 906 && minor_version >= 3) ||
+			(major_version == 905 && minor_version >= 7) ||
+			(major_version == 904 && minor_version >= 12) ||
+			(major_version == 903 && minor_version >= 17) ||
+			(major_version == 902 && minor_version >= 21))
+			set_acl_ok = true;
+	}
+#endif
+
 	/* Define custom GUC variables. */
 	DefineCustomBoolVariable("pg_dbms_stats.use_locked_stats",
 							 "Enable user defined statistics.",
@@ -228,6 +292,103 @@ _PG_fini(void)
 	/* A function to unregister callback for relcache is NOT provided. */
 }
 
+/*
+ * Function to convert from any array from dbms_stats.anyarray.
+ */
+Datum
+dbms_stats_anyary_anyary(PG_FUNCTION_ARGS)
+{
+  ArrayType *arr = PG_GETARG_ARRAYTYPE_P(0);
+  if (ARR_NDIM(arr) != 1)
+	  elog(ERROR, "array must be one-dimentional.");
+
+  PG_RETURN_ARRAYTYPE_P(arr);
+}
+
+/*
+ * Function to check if the type can have statistics.
+ */
+Datum
+dbms_stats_type_is_analyzable(PG_FUNCTION_ARGS)
+{
+	Oid typid = PG_GETARG_OID(0);
+	Oid	eqopr;
+
+	if (!OidIsValid(typid))
+		PG_RETURN_BOOL(false);
+
+	get_sort_group_operators(typid, false, false, false,
+							 NULL, &eqopr, NULL,
+							 NULL);
+	PG_RETURN_BOOL(OidIsValid(eqopr));
+}
+
+/*
+ * Function to get base type of the value of the type dbms_stats.anyarray.
+ */
+Datum
+dbms_stats_anyarray_basetype(PG_FUNCTION_ARGS)
+{
+	ArrayType  *arr = PG_GETARG_ARRAYTYPE_P(0);
+	Oid			elemtype = arr->elemtype;
+	HeapTuple	tp;
+	Form_pg_type typtup;
+	Name		result;
+
+	if (!OidIsValid(elemtype))
+		elog(ERROR, "invalid base type oid: %u", elemtype);
+
+	tp = SearchSysCache1(TYPEOID, ObjectIdGetDatum(elemtype));
+	if (!HeapTupleIsValid(tp))  /* I trust you. */
+		elog(ERROR, "invalid base type oid: %u", elemtype);
+
+	typtup = (Form_pg_type) GETSTRUCT(tp);
+	result = (Name) palloc0(NAMEDATALEN);
+	StrNCpy(NameStr(*result), NameStr(typtup->typname), NAMEDATALEN);
+
+	ReleaseSysCache(tp);
+	PG_RETURN_NAME(result);
+}
+
+/*
+ * Find and store the owner of the dummy statistics table.
+ *
+ * We will access statistics tables using this owner
+ */
+static Oid
+get_stats_table_owner(void)
+{
+	HeapTuple tp;
+
+	if (!OidIsValid(stats_table_owner))
+	{
+		tp = SearchSysCache2(RELNAMENSP,
+					 PointerGetDatum(RELSTAT_TBLNAME),
+					 ObjectIdGetDatum(get_namespace_oid(NSPNAME, false)));
+		if (!HeapTupleIsValid(tp))
+			elog(ERROR, "table \"%s.%s\" not found in pg_class",
+				 NSPNAME, RELSTAT_TBLNAME);
+		stats_table_owner =	((Form_pg_class) GETSTRUCT(tp))->relowner;
+		if (!OidIsValid(stats_table_owner))
+			elog(ERROR, "owner uid of table \"%s.%s\" is invalid",
+				 NSPNAME, RELSTAT_TBLNAME);
+		ReleaseSysCache(tp);
+
+		tp = SearchSysCache1(AUTHOID, ObjectIdGetDatum(stats_table_owner));
+		if (!HeapTupleIsValid(tp))
+		{
+			elog(ERROR,
+				 "role id %u for the owner of the relation \"%s.%s\"is invalid",
+				 stats_table_owner, NSPNAME, RELSTAT_TBLNAME);
+		}
+		/* This will be done once for the session, so not pstrdup. */
+		stats_table_owner_name =
+			strdup(NameStr(((Form_pg_authid) GETSTRUCT(tp))->rolname));
+		ReleaseSysCache(tp);
+	}
+	return stats_table_owner;
+}
+
 /*
  * Store heap tuple header into given heap tuple.
  */
@@ -599,7 +760,7 @@ dbms_stats_is_system_schema_internal(char *schema_name)
 	if (strcmp(schema_name, "pg_catalog") == 0 ||
 		strcmp(schema_name, "pg_toast") == 0 ||
 		strcmp(schema_name, "information_schema") == 0 ||
-		strcmp(schema_name, "dbms_stats") == 0)
+		strcmp(schema_name, NSPNAME) == 0)
 		return true;
 
 	return false;
@@ -693,15 +854,8 @@ dbms_stats_get_relation_info(PlannerInfo *root,
 	 * calculation.
 	 */
 	if (!inhparent)
-	{
-#if PG_VERSION_NUM >= 90200
 		get_merged_relation_stats(relid, &rel->pages, &rel->tuples,
 								  &rel->allvisfrac, true);
-#else
-		get_merged_relation_stats(relid, &rel->pages, &rel->tuples,
-								  &allvisfrac, true);
-#endif
-	}
 	else
 		return;
 
@@ -773,9 +927,24 @@ dbms_stats_get_relation_stats(PlannerInfo *root,
 
 		tuple = get_merged_column_stats(rte->relid, attnum, rte->inh);
 		vardata->statsTuple = tuple;
-		if (tuple != NULL)
+		if (HeapTupleIsValid(tuple))
 		{
 			vardata->freefunc = FreeHeapTuple;
+
+#ifdef PGDS_HAVE_ACL_OK
+			/*
+			 * set acl_ok if required. See the definition of set_acl_ok for
+			 * details.
+			 */
+			if (set_acl_ok)
+			 {
+				 vardata->acl_ok =
+					 (pg_class_aclcheck(rte->relid, GetUserId(),
+										ACL_SELECT) == ACLCHECK_OK) ||
+					 (pg_attribute_aclcheck(rte->relid, attnum, GetUserId(),
+											ACL_SELECT) == ACLCHECK_OK);
+			 }
+#endif
 			return true;
 		}
 	}
@@ -798,19 +967,74 @@ dbms_stats_get_index_stats(PlannerInfo *root,
 						   AttrNumber indexattnum,
 						   VariableStatData *vardata)
 {
-	if (pg_dbms_stats_use_locked_stats)
-	{
-		HeapTuple	tuple;
+	HeapTuple	tuple;
 
-		tuple = get_merged_column_stats(indexOid, indexattnum, false);
-		vardata->statsTuple = tuple;
-		if (tuple != NULL)
+	if (!pg_dbms_stats_use_locked_stats)
+		goto next_plugin;
+
+	tuple = get_merged_column_stats(indexOid, indexattnum, false);
+	vardata->statsTuple = tuple;
+	if (tuple == NULL)
+		goto next_plugin;
+
+	vardata->freefunc = FreeHeapTuple;
+			
+#ifdef PGDS_HAVE_ACL_OK
+	/*
+	 * set acl_ok if required. See the definition of set_acl_ok for details.
+	 */
+	if (set_acl_ok)
+	{
+		/*
+		 * XXX: we had to scan the whole the rel array since we got
+		 * only the oid of the index.
+		 */
+		int i;
+
+		/* don't stop by this misassumption */
+		if (root->simple_rel_array == NULL)
 		{
-			vardata->freefunc = FreeHeapTuple;
-			return true;
+			elog(WARNING, "pg_dbms_stats internal error. relation has not been set up. index %d ignored", indexOid);
+			goto next_plugin;
+		}
+
+		/*
+		 * scan over simple_rel_array_size to find the owner relation of the
+		 * index that its oid is indexOid
+		 */
+		for (i = 1 ; i < root->simple_rel_array_size ; i++)
+		{
+			ListCell *lc;
+
+			foreach (lc, root->simple_rel_array[i]->indexlist)
+			{
+				IndexOptInfo *index = (IndexOptInfo *) lfirst(lc);
+				RangeTblEntry *rte;
+
+				if (index->indexoid != indexOid)
+					continue;
+
+				/* This relation is the owner of the given index, go ahead */
+				rte = planner_rt_fetch(index->rel->relid, root);
+
+				/* don't stop by this error */
+				if (rte->rtekind != RTE_RELATION)
+				{
+					elog(WARNING, "pg_dbms_stats internal error. index %d is owned by a non-relation", indexOid);
+					goto next_plugin;
+				}
+
+				vardata->acl_ok =
+					(pg_class_aclcheck(rte->relid, GetUserId(),
+									   ACL_SELECT) == ACLCHECK_OK);
+				break;
+			}
 		}
 	}
+#endif
+	return true;
 
+next_plugin:
 	if (prev_get_index_stats)
 		return prev_get_index_stats(root, indexOid, indexattnum, vardata);
 	else
@@ -865,7 +1089,7 @@ get_merged_relation_stats(Oid relid, BlockNumber *pages, double *tuples,
 		 * eliminates excessive SPI calls below. Negative caches will be
 		 * invalidated again on invalidation of system relation cache, which
 		 * occur on modification of the dummy stats tables
-		 * dbms_stats._relation_stats_locked and _column_stats_locked.
+		 * dbms_stats.relation_stats_locked and column_stats_locked.
 		 */
 		if (entry->relpages == InvalidBlockNumber)
 			return;
@@ -882,7 +1106,6 @@ get_merged_relation_stats(Oid relid, BlockNumber *pages, double *tuples,
 		PG_TRY();
 		{
 			++nested_level;
-
 			SPI_connect();
 
 			/*
@@ -921,11 +1144,9 @@ get_merged_relation_stats(Oid relid, BlockNumber *pages, double *tuples,
 				val = get_binary_datum(3, &isnull);
 				entry->curpages = isnull ? InvalidBlockNumber :
 					(BlockNumber) DatumGetInt32(val);
-#if PG_VERSION_NUM >= 90200
 				val = get_binary_datum(4, &isnull);
 				entry->relallvisible = isnull ? form->relallvisible :
 					(BlockNumber) DatumGetInt32(val);
-#endif
 
 				ReleaseSysCache(tuple);
 			}
@@ -963,9 +1184,7 @@ get_merged_relation_stats(Oid relid, BlockNumber *pages, double *tuples,
 	rel = relation_open(relid, NoLock);
 	rel->rd_rel->relpages = entry->relpages;
 	rel->rd_rel->reltuples = entry->reltuples;
-#if PG_VERSION_NUM >= 90200
 	rel->rd_rel->relallvisible = entry->relallvisible;
-#endif
 	dbms_stats_estimate_rel_size(rel, NULL, pages, tuples, allvisfrac,
 								 entry->curpages);
 	relation_close(rel, NoLock);
@@ -1066,7 +1285,7 @@ get_merged_column_stats(Oid relid, AttrNumber attnum, bool inh)
 				tuple = NULL;
 
 			/* Cache merged result for subsequent calls. */
-			tuple = column_cache_enter(relid, attnum, tuple);
+			tuple = column_cache_enter(relid, attnum, inh, tuple);
 
 			/* Return system stats if the merging results in failure. */
 			if (!HeapTupleIsValid(tuple))
@@ -1118,7 +1337,7 @@ column_cache_search(Oid relid, AttrNumber attnum, bool inh, bool *negative)
 	{
 		StatsColumnEntry *ent = (StatsColumnEntry*) lfirst (lc);
 
-		if (ent->attnum != attnum) continue;
+		if (ent->attnum != attnum || ent->inh != inh) continue;
 
 		if (ent->negative)
 		{
@@ -1126,15 +1345,8 @@ column_cache_search(Oid relid, AttrNumber attnum, bool inh, bool *negative)
 			*negative = true;
 			return NULL;
 		}
-		else
-		{
-			HeapTuple	tuple = (HeapTuple) ent->tuple;
-			Form_pg_statistic form = get_pg_statistic(tuple);
 
-			/* Find statistic of the given column from the cache. */
-			if (form->stainherit == inh)
-				return tuple;
-		}
+		return ent->tuple;
 	}
 
 	return NULL;	/* Not yet registered. */
@@ -1146,7 +1358,7 @@ column_cache_search(Oid relid, AttrNumber attnum, bool inh, bool *negative)
  * table definition have been changed.
  */
 static HeapTuple
-column_cache_enter(Oid relid, int32 attnum, HeapTuple tuple)
+column_cache_enter(Oid relid, int32 attnum, bool inh, HeapTuple tuple)
 {
 	MemoryContext	oldcontext;
 	StatsColumnEntry *newcolent;
@@ -1166,6 +1378,8 @@ column_cache_enter(Oid relid, int32 attnum, HeapTuple tuple)
 	oldcontext = MemoryContextSwitchTo(CacheMemoryContext);
 	newcolent = (StatsColumnEntry*)palloc(sizeof(StatsColumnEntry));
 	newcolent->attnum = attnum;
+	newcolent->inh = inh;
+
 	if (HeapTupleIsValid(tuple))
 	{
 		newcolent->negative = false;
@@ -1202,27 +1416,54 @@ execute_plan(SPIPlanPtr *plan,
 	int		nargs;
 	Datum	values[3];
 	bool	nulls[3] = { false, false, false };
+	Oid			save_userid;
+	int			save_sec_context;
 
+	/* XXXX: this works for now but should be fixed later.. */
 	nargs = (attnum ? 3 : 1);
 
-	/* When plan is not given, create plan from query string at first. */
-	if (*plan == NULL)
+	/*
+	 * The dummy statistics table allows access from no one other than its
+	 * owner or superuser.
+	 */
+	GetUserIdAndSecContext(&save_userid, &save_sec_context);
+	SetUserIdAndSecContext(get_stats_table_owner(),
+						   save_sec_context | SECURITY_LOCAL_USERID_CHANGE);
+
+	PG_TRY();
 	{
-		SPIPlanPtr p;
-		p = SPI_prepare(query, nargs, argtypes);
-		if (p == NULL)
-			elog(ERROR, "pg_dbms_stats: SPI_prepare => %d", SPI_result);
-		*plan = SPI_saveplan(p);
-		SPI_freeplan(p);
+		/* Create plan from the query if not yet. */
+		if (*plan == NULL)
+		{
+			*plan = SPI_prepare(query, nargs, argtypes);
+			if (*plan == NULL)
+				elog(ERROR,
+					 "pg_dbms_stats: SPI_prepare() failed. result = %d",
+					 SPI_result);
+
+			SPI_keepplan(*plan);
+		}
+
+		values[0] = ObjectIdGetDatum(relid);
+		values[1] = Int16GetDatum(attnum ? *attnum : 0);
+		values[2] = BoolGetDatum(inh);
+
+		ret = SPI_execute_plan(*plan, values, nulls, true, 1);
 	}
+	PG_CATCH();
+	{
+		SetUserIdAndSecContext(save_userid, save_sec_context);
+		if (geterrcode() == ERRCODE_INSUFFICIENT_PRIVILEGE)
+			errdetail("dbms_stats could not access the object as the role \"%s\".",
+				stats_table_owner_name);
+		errhint("Check your settings of pg_dbms_stats.");
+		PG_RE_THROW();
+	}
+	PG_END_TRY();
 
-	values[0] = ObjectIdGetDatum(relid);
-	values[1] = Int16GetDatum(attnum ? *attnum : 0);
-	values[2] = BoolGetDatum(inh);
-
-	ret = SPI_execute_plan(*plan, values, nulls, true, 1);
+	SetUserIdAndSecContext(save_userid, save_sec_context);
 	if (ret != SPI_OK_SELECT)
-		elog(ERROR, "pg_dbms_stats: SPI_execute_plan => %d", ret);
+		elog(ERROR, "pg_dbms_stats: SPI_execute_plan() returned %d", ret);
 
 	return SPI_processed > 0;
 }
@@ -1323,12 +1564,12 @@ init_rel_stats_entry(StatsRelationEntry *entry, Oid relid)
  * the attribute widths for estimation purposes.
  *
  * Note: This function is copied from plancat.c in core source tree of version
- * 9.2, and customized for pg_dbms_stats.  Changes from orignal one are:
+ * 9.2, and customized for pg_dbms_stats.  Changes from original one are:
  *   - rename by prefixing dbms_stats_
  *   - add 3 parameters (relpages, reltuples, curpage) to pass dummy curpage
  *     values.
  *   - Get current # of pages only when supplied curpages is InvalidBlockNumber
- *   - get franction of all-visible-pages
+ *   - get fraction of all-visible-pages
  */
 static void
 dbms_stats_estimate_rel_size(Relation rel, int32 *attr_widths,
@@ -1400,11 +1641,8 @@ dbms_stats_estimate_rel_size(Relation rel, int32 *attr_widths,
 			/* coerce values in pg_class to more desirable types */
 			relpages = (BlockNumber) rel->rd_rel->relpages;
 			reltuples = (double) rel->rd_rel->reltuples;
-#if PG_VERSION_NUM >= 90200
 			relallvisible = (BlockNumber) rel->rd_rel->relallvisible;
-#else
-			relallvisible = 0;
-#endif
+
 			/*
 			 * If it's an index, discount the metapage while estimating the
 			 * number of tuples.  This is a kluge because it assumes more than
