@@ -1,5 +1,5 @@
 \pset null '(null)'
-
+\set SHOW_CONTEXT never
 /*
  * No.2-3 dbms_stats.backup_history_id_seq
  */
@@ -23,11 +23,11 @@ DELETE FROM dbms_stats.backup_history;
 /*
  * No.3-1 dbms_stats.use_locked_stats
  */
-DELETE FROM dbms_stats._relation_stats_locked;
+DELETE FROM dbms_stats.relation_stats_locked;
 EXPLAIN (costs false) SELECT * FROM s0.st2 WHERE id < 1;
 SELECT dbms_stats.lock_table_stats('s0.st2'::regclass);
-UPDATE dbms_stats._relation_stats_locked SET curpages = 10000;
-VACUUM dbms_stats._relation_stats_locked;  -- in order to avoid auto vacuum
+UPDATE dbms_stats.relation_stats_locked SET curpages = 10000;
+VACUUM ANALYZE;
 -- No.3-1-1
 SET pg_dbms_stats.use_locked_stats TO ON;
 EXPLAIN (costs false) SELECT * FROM s0.st2 WHERE id < 1;
@@ -35,6 +35,7 @@ EXPLAIN (costs false) SELECT * FROM s0.st2 WHERE id < 1;
 SET pg_dbms_stats.use_locked_stats TO OFF;
 EXPLAIN (costs false) SELECT * FROM s0.st2 WHERE id < 1;
 -- No.3-1-3
+/* Reconnection as regular user */
 \c - regular_user
 SHOW pg_dbms_stats.use_locked_stats;
 SET pg_dbms_stats.use_locked_stats TO OFF;
@@ -43,8 +44,9 @@ EXPLAIN (costs false) SELECT * FROM s0.st2 WHERE id < 1;
 RESET pg_dbms_stats.use_locked_stats;
 EXPLAIN (costs false) SELECT * FROM s0.st2 WHERE id < 1;
 -- clean up
+/* Reconnection as super user */
 \c - super_user
-DELETE FROM dbms_stats._relation_stats_locked;
+DELETE FROM dbms_stats.relation_stats_locked;
 
 /*
  * No.4-1 DATA TYPE dbms_stats.anyarray
@@ -55,8 +57,10 @@ ANALYZE st3;
 SELECT staattnum, stavalues1 FROM pg_statistic
  WHERE starelid = 'public.st3'::regclass
  ORDER BY staattnum;
-\copy (SELECT stavalues1::dbms_stats.anyarray FROM dbms_stats.column_stats_effective WHERE starelid = 'st3'::regclass) TO 'results/anyarray_test.cp' binary
-CREATE TABLE st4 (arr dbms_stats.anyarray);
+\copy (SELECT stavalues1::dbms_stats.anyarray FROM dbms_stats.column_stats_effective WHERE starelid = 'st3'::regclass ORDER BY staattnum) TO 'results/anyarray_test.cp' binary
+SET client_min_messages TO WARNING;
+CREATE TABLE st4 (arr dbms_stats.anyarray, ord serial);
+SET client_min_messages TO DEFAULT;
 
 SELECT t.typname, n.nspname,
        t.typlen, t.typbyval, t.typtype,
@@ -71,7 +75,7 @@ SELECT t.typname, n.nspname,
  ORDER BY t.typname;
 -- No.4-1-1
 INSERT INTO st4 VALUES(NULL);
-SELECT * FROM st4;
+SELECT * FROM st4 ORDER BY ord;
 -- No.4-1-2
 DELETE FROM st4;
 SELECT stavalues1::dbms_stats.anyarray
@@ -84,7 +88,7 @@ INSERT INTO st4
        FROM dbms_stats.column_stats_effective
       WHERE starelid = 'st3'::regclass
         AND staattnum = 1;
-SELECT * FROM st4;
+SELECT * FROM st4 ORDER BY ord;
 -- No.4-1-3
 DELETE FROM st4;
 SELECT stavalues1::dbms_stats.anyarray
@@ -97,7 +101,7 @@ INSERT INTO st4
        FROM dbms_stats.column_stats_effective
       WHERE starelid = 'st3'::regclass
         AND staattnum = 2;
-SELECT * FROM st4;
+SELECT * FROM st4 ORDER BY ord;
 -- No.4-1-4
 DELETE FROM st4;
 SELECT stavalues1::dbms_stats.anyarray
@@ -110,7 +114,7 @@ INSERT INTO st4
        FROM dbms_stats.column_stats_effective
       WHERE starelid = 'st3'::regclass
         AND staattnum = 1;
-SELECT * FROM st4;
+SELECT * FROM st4 ORDER BY ord;
 -- No.4-1-5
 DELETE FROM st4;
 SELECT stavalues1::dbms_stats.anyarray
@@ -123,14 +127,15 @@ INSERT INTO st4
        FROM dbms_stats.column_stats_effective
       WHERE starelid = 'st3'::regclass
         AND staattnum = 3;
-SELECT * FROM st4;
+SELECT * FROM st4 ORDER BY ord;
 -- No.4-1-6
 DELETE FROM st4;
 SELECT stavalues1::dbms_stats.anyarray
   FROM dbms_stats.column_stats_effective
- WHERE starelid = 'st3'::regclass;
+ WHERE starelid = 'st3'::regclass
+ ORDER BY staattnum;
 \copy st4(arr) FROM 'results/anyarray_test.cp' binary
-SELECT * FROM st4;
+SELECT * FROM st4 ORDER BY ord;
 -- clean up
 DROP TABLE st3;
 DROP TABLE st4;
@@ -179,38 +184,38 @@ DROP TRIGGER invalidate_cache3 ON pt0;
 
 -- No.5-2-5
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-INSERT INTO dbms_stats._relation_stats_locked (relid, relname) VALUES (0, 'dummy');
-INSERT INTO dbms_stats._column_stats_locked (starelid, staattnum, stainherit)
+INSERT INTO dbms_stats.relation_stats_locked (relid, relname) VALUES (0, 'dummy');
+INSERT INTO dbms_stats.column_stats_locked (starelid, staattnum, stainherit)
     VALUES (0, 1, true);
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-2-6
-INSERT INTO dbms_stats._column_stats_locked (starelid, staattnum, stainherit)
+INSERT INTO dbms_stats.column_stats_locked (starelid, staattnum, stainherit)
     VALUES ('st1_idx'::regclass, 1, true);
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-2-7
-INSERT INTO dbms_stats._column_stats_locked (starelid, staattnum, stainherit)
+INSERT INTO dbms_stats.column_stats_locked (starelid, staattnum, stainherit)
     VALUES ('complex'::regclass, 1, true);
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-2-9
-UPDATE dbms_stats._column_stats_locked SET stanullfrac = 1
+UPDATE dbms_stats.column_stats_locked SET stanullfrac = 1
  WHERE starelid = 'st1'::regclass
    AND staattnum = 1
    AND stainherit = false;
-VACUUM dbms_stats._relation_stats_locked;  -- in order to avoid auto vacuum
+VACUUM ANALYZE;
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-2-10
-DELETE FROM dbms_stats._column_stats_locked
+DELETE FROM dbms_stats.column_stats_locked
  WHERE starelid = 'st1'::regclass
    AND staattnum = 1
    AND stainherit = false;
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-2-8
-INSERT INTO dbms_stats._column_stats_locked
+INSERT INTO dbms_stats.column_stats_locked
     (starelid, staattnum, stainherit, stanullfrac)
     VALUES ('st1'::regclass, 1, false, 1);
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
@@ -219,9 +224,9 @@ EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 PREPARE p2 AS SELECT str FROM st1 WHERE lower(str) IS NULL;
 EXPLAIN (costs false) SELECT str FROM st1 WHERE lower(str) IS NULL;
 EXPLAIN (costs false) EXECUTE p2;
-INSERT INTO dbms_stats._relation_stats_locked (relid, relname)
+INSERT INTO dbms_stats.relation_stats_locked (relid, relname)
     VALUES ('st1_exp'::regclass, 'dummy');
-INSERT INTO dbms_stats._column_stats_locked
+INSERT INTO dbms_stats.column_stats_locked
     (starelid, staattnum, stainherit, stanullfrac)
     VALUES ('st1_exp'::regclass, 1, false, 1);
 EXPLAIN (costs false) SELECT str FROM st1 WHERE lower(str) IS NULL;
@@ -275,37 +280,37 @@ DROP TRIGGER invalidate_cache3 ON pt0;
 SELECT dbms_stats.unlock_database_stats();
 SELECT dbms_stats.lock_table_stats('st1');
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-INSERT INTO dbms_stats._relation_stats_locked (relid, relname) VALUES (0, 'dummy');
+INSERT INTO dbms_stats.relation_stats_locked (relid, relname) VALUES (0, 'dummy');
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-3-6
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-INSERT INTO dbms_stats._relation_stats_locked (relid, relname)
+INSERT INTO dbms_stats.relation_stats_locked (relid, relname)
     VALUES ('st1_idx'::regclass, 'st1_idx');
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-3-7
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-INSERT INTO dbms_stats._relation_stats_locked (relid, relname)
+INSERT INTO dbms_stats.relation_stats_locked (relid, relname)
     VALUES ('complex'::regclass, 'complex');
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-3-9
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-UPDATE dbms_stats._relation_stats_locked SET curpages = 1
+UPDATE dbms_stats.relation_stats_locked SET curpages = 1
  WHERE relid = 'st1'::regclass;
-VACUUM dbms_stats._relation_stats_locked;  -- in order to avoid auto vacuum
+VACUUM ANALYZE;
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-3-10
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-DELETE FROM dbms_stats._relation_stats_locked
+DELETE FROM dbms_stats.relation_stats_locked
  WHERE relid = 'st1'::regclass;
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-3-8
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-INSERT INTO dbms_stats._relation_stats_locked (relid, relname, curpages)
+INSERT INTO dbms_stats.relation_stats_locked (relid, relname, curpages)
     VALUES ('st1'::regclass, 'st1', 1);
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
@@ -314,26 +319,28 @@ SELECT dbms_stats.unlock_database_stats();
 SELECT dbms_stats.lock_table_stats('st1');
 SELECT relname, curpages FROM dbms_stats.relation_stats_locked
  WHERE relid = 'st1'::regclass;
-SELECT pg_sleep(0.5);
-SELECT reset_stat_and_cache();
-VACUUM dbms_stats._relation_stats_locked;  -- in order to avoid auto vacuum
-UPDATE dbms_stats._relation_stats_locked SET curpages = 1000
+SELECT pg_sleep(0.7);
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
+VACUUM ANALYZE;
+UPDATE dbms_stats.relation_stats_locked SET curpages = 1000
  WHERE relid = 'st1_exp'::regclass;
-SELECT pg_sleep(0.5);
+SELECT pg_sleep(0.7);
 SELECT * FROM lockd_io;
-SELECT reset_stat_and_cache();
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
 SELECT relname, curpages FROM dbms_stats.relation_stats_locked
  WHERE relid = 'st1'::regclass;
-SELECT pg_sleep(0.5);
+SELECT pg_sleep(0.7);
 SELECT * FROM lockd_io;
 
 /*
  * No.5-4 StatsCacheRelCallback
  */
 -- No.5-4-1
-UPDATE dbms_stats._relation_stats_locked SET curpages = 1
+UPDATE dbms_stats.relation_stats_locked SET curpages = 1
  WHERE relid = 'st1'::regclass;
-VACUUM dbms_stats._relation_stats_locked;  -- in order to avoid auto vacuum
+VACUUM ANALYZE;
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 \c
 SET pg_dbms_stats.use_locked_stats to NO;
@@ -345,17 +352,17 @@ SELECT dbms_stats.unlock_database_stats();
 SELECT dbms_stats.lock_table_stats('st1');
 -- No.5-4-3
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-\! psql contrib_regression -c "UPDATE dbms_stats._column_stats_locked SET stanullfrac = 1 WHERE starelid = 'st1'::regclass"
+\! psql contrib_regression -c "UPDATE dbms_stats.column_stats_locked SET stanullfrac = 1 WHERE starelid = 'st1'::regclass"
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-4-4
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-\! psql contrib_regression -c "DELETE FROM dbms_stats._column_stats_locked WHERE starelid = 'st1'::regclass"
+\! psql contrib_regression -c "DELETE FROM dbms_stats.column_stats_locked WHERE starelid = 'st1'::regclass"
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 -- No.5-4-2
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
-\! psql contrib_regression -c "INSERT INTO dbms_stats._column_stats_locked (starelid, staattnum, stainherit, stanullfrac) VALUES ('st1'::regclass, 1, false, 1)"
+\! psql contrib_regression -c "INSERT INTO dbms_stats.column_stats_locked (starelid, staattnum, stainherit, stanullfrac) VALUES ('st1'::regclass, 1, false, 1)"
 EXPLAIN (costs false) SELECT * FROM st1 WHERE val IS NULL;
 
 SELECT dbms_stats.unlock_database_stats();
@@ -366,30 +373,35 @@ INSERT INTO s0.droptest VALUES (1),(2),(3);
 VACUUM ANALYZE;
 SELECT * FROM s0.droptest
  WHERE id = 1;
-SELECT pg_sleep(0.5);
-SELECT reset_stat_and_cache();
+SELECT pg_sleep(0.7);
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
 ALTER TABLE s0.droptest RENAME TO test;
-SELECT pg_sleep(0.5);
+SELECT pg_sleep(0.7);
 SELECT * FROM lockd_io;
-SELECT reset_stat_and_cache();
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
 SELECT * FROM s0.test
  WHERE id = 1;
-SELECT pg_sleep(0.5);
+SELECT pg_sleep(0.7);
 SELECT * FROM lockd_io;
 ALTER TABLE s0.test RENAME TO droptest;
 
 -- No.5-4-6
+VACUUM ANALYZE;
 SELECT * FROM s0.droptest
  WHERE id = 1;
-SELECT pg_sleep(0.5);
-SELECT reset_stat_and_cache();
+SELECT pg_sleep(0.7);
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
 ALTER TABLE s0.droptest RENAME id TO test;
-SELECT pg_sleep(0.5);
+SELECT pg_sleep(0.7);
 SELECT * FROM lockd_io;
-SELECT reset_stat_and_cache();
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
 SELECT * FROM s0.droptest
  WHERE test = 1;
-SELECT pg_sleep(0.5);
+SELECT pg_sleep(0.7);
 SELECT * FROM lockd_io;
 ALTER TABLE s0.droptest RENAME test TO id;
 
@@ -397,12 +409,14 @@ ALTER TABLE s0.droptest RENAME test TO id;
 INSERT INTO s0.droptest VALUES (4);
 SELECT * FROM s0.droptest
  WHERE id = 1;
-SELECT pg_sleep(0.5);
-SELECT reset_stat_and_cache();
+SELECT pg_sleep(0.7);
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
 ANALYZE;
-SELECT pg_sleep(0.5);
+SELECT pg_sleep(0.7);
 SELECT * FROM lockd_io;
-SELECT reset_stat_and_cache();
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
 SELECT * FROM s0.droptest
  WHERE id = 1;
 SELECT pg_sleep(1.0);
@@ -413,15 +427,17 @@ DELETE FROM s0.droptest;
 INSERT INTO s0.droptest VALUES (4),(5);
 SELECT * FROM s0.droptest
  WHERE id = 4;
-SELECT pg_sleep(0.5);
-SELECT reset_stat_and_cache();
-VACUUM;
-SELECT pg_sleep(0.5);
+SELECT pg_sleep(0.7);
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
+VACUUM ANALYZE;
+SELECT pg_sleep(0.7);
 SELECT * FROM lockd_io;
-SELECT reset_stat_and_cache();
+SELECT load_merged_stats();
+SELECT pg_stat_reset();
 SELECT * FROM s0.droptest
  WHERE id = 4;
-SELECT pg_sleep(0.5);
+SELECT pg_sleep(0.7);
 SELECT * FROM lockd_io;
 
 -- clean up
@@ -839,6 +855,7 @@ LANGUAGE plpgsql;
 /*
  * No.10-3 dbms_stats.restore_table_stats(regclass, as_of_timestamp)
  */
+\set VERBOSITY terse
 -- No.10-3-1
 SELECT dbms_stats.restore_table_stats('s0.st0', '2012-02-29 23:59:57');
 -- No.10-3-2
@@ -850,6 +867,7 @@ SELECT dbms_stats.restore_table_stats('s0.st0', '2012-01-01 00:00:00');
 SELECT dbms_stats.restore_table_stats('s0.st0', '2012-02-29 23:59:57');
 -- No.10-3-6
 SELECT dbms_stats.restore_table_stats('st0', '2012-02-29 23:59:57');
+\set VERBOSITY default
 -- No.10-3-7
 SELECT dbms_stats.restore_table_stats('s00.s0', '2012-02-29 23:59:57');
 /*
@@ -874,6 +892,7 @@ LANGUAGE plpgsql;
 /*
  * No.10-4 dbms_stats.restore_table_stats(schemaname, tablename, as_of_timestamp)
  */
+\set VERBOSITY terse
 -- No.10-4-1
 SELECT dbms_stats.restore_table_stats('s0', 'st0', '2012-02-29 23:59:57');
 DROP FUNCTION dbms_stats.restore_table_stats(regclass,
@@ -896,6 +915,7 @@ SELECT dbms_stats.restore_column_stats('s0.st0', 'id', '2012-01-01 00:00:00');
 SELECT dbms_stats.restore_column_stats('s0.st0', 'id', '2012-02-29 23:59:57');
 -- No.10-5-6
 SELECT dbms_stats.restore_column_stats('st0', 'id', '2012-02-29 23:59:57');
+\set VERBOSITY default
 -- No.10-5-7
 SELECT dbms_stats.restore_column_stats('s00.s0', 'id', '2012-02-29 23:59:57');
 
@@ -904,8 +924,9 @@ SELECT dbms_stats.restore_column_stats('s00.s0', 'id', '2012-02-29 23:59:57');
  *        schemaname, tablename, attname, as_of_timestamp)
  */
 -- No.10-6-1
+\set VERBOSITY terse
 SELECT dbms_stats.restore_column_stats('s0', 'st0', 'id', '2012-02-29 23:59:57');
-
+\set VERBOSITY default
 /*
  * No.15-1 dbms_stats.purge_stats
  */
@@ -914,16 +935,22 @@ SELECT * FROM dbms_stats.backup_history;
 BEGIN;
 SELECT relation::regclass, mode
   FROM pg_locks
-  WHERE relation::regclass::text LIKE 'dbms_stats.\_%\_locked'
+  WHERE
+  	(relation::regclass::text LIKE 'dbms_stats.\_%\_locked'
      OR relation::regclass::text LIKE 'dbms_stats.backup_history'
-     OR relation::regclass::text LIKE 'dbms_stats.%\_backup'
+     OR relation::regclass::text LIKE 'dbms_stats.%\_backup')
+  AND
+    mode <> 'ShareUpdateExclusiveLock'
   ORDER BY relation::regclass::text, mode;
 SELECT id, unit, comment FROM dbms_stats.purge_stats(2);
 SELECT relation::regclass, mode
   FROM pg_locks
-  WHERE relation::regclass::text LIKE 'dbms_stats.\_%\_locked'
+  WHERE
+  	(relation::regclass::text LIKE 'dbms_stats.\_%\_locked'
      OR relation::regclass::text LIKE 'dbms_stats.backup_history'
-     OR relation::regclass::text LIKE 'dbms_stats.%\_backup'
+     OR relation::regclass::text LIKE 'dbms_stats.%\_backup')
+  AND
+    mode <> 'ShareUpdateExclusiveLock'
   ORDER BY relation::regclass::text, mode;
 COMMIT;
 SELECT * FROM dbms_stats.backup_history;
@@ -1028,10 +1055,11 @@ SELECT dbms_stats.restore_database_stats('2012-02-29 23:59:57');
 -- No.10-2-4
 SELECT dbms_stats.restore_schema_stats('s0', '2012-02-29 23:59:57');
 -- No.10-3-4
+\set VERBOSITY terse
 SELECT dbms_stats.restore_table_stats('s0.st0', '2012-02-29 23:59:57');
 -- No.10-5-4
 SELECT dbms_stats.restore_column_stats('s0.st0', 'id', '2012-02-29 23:59:57');
-
+\set VERBOSITY default
 /*
  * Delete stab function dbms_stats.restore
  */
@@ -1050,27 +1078,27 @@ ANALYZE clean_test;
 SELECT dbms_stats.lock_table_stats('clean_test');
 SELECT count(*) FROM dbms_stats.relation_stats_locked;
 SELECT count(*) FROM dbms_stats.column_stats_locked;
-SELECT dbms_stats.clean_up_stats();
+SELECT dbms_stats.clean_up_stats() ORDER BY 1;
 SELECT count(*) FROM dbms_stats.relation_stats_locked;
 SELECT count(*) FROM dbms_stats.column_stats_locked;
 -- No.18-1-2
 -- No.18-1-7
-DELETE FROM dbms_stats._relation_stats_locked;
+DELETE FROM dbms_stats.relation_stats_locked;
 SELECT count(*) FROM dbms_stats.relation_stats_locked;
 SELECT count(*) FROM dbms_stats.column_stats_locked;
-SELECT dbms_stats.clean_up_stats();
+SELECT dbms_stats.clean_up_stats() ORDER BY 1;
 SELECT count(*) FROM dbms_stats.relation_stats_locked;
 SELECT count(*) FROM dbms_stats.column_stats_locked;
 -- No.18-1-3
 SELECT dbms_stats.lock_table_stats('clean_test');
 DROP TABLE clean_test;
 SELECT count(*) FROM dbms_stats.relation_stats_locked;
-SELECT dbms_stats.clean_up_stats();
+SELECT dbms_stats.clean_up_stats() ORDER BY 1;
 SELECT count(*) FROM dbms_stats.relation_stats_locked;
 -- No.18-1-4
-DELETE FROM dbms_stats._relation_stats_locked;
+DELETE FROM dbms_stats.relation_stats_locked;
 SELECT count(*) FROM dbms_stats.relation_stats_locked;
-SELECT dbms_stats.clean_up_stats();
+SELECT dbms_stats.clean_up_stats() ORDER BY 1;
 SELECT count(*) FROM dbms_stats.relation_stats_locked;
 -- No.18-1-6
 CREATE TABLE clean_test(id integer, num integer);
@@ -1079,7 +1107,7 @@ ANALYZE clean_test;
 SELECT dbms_stats.lock_table_stats('clean_test');
 ALTER TABLE clean_test DROP COLUMN num;
 ALTER TABLE clean_test ADD num integer;
-UPDATE dbms_stats._column_stats_locked
+UPDATE dbms_stats.column_stats_locked
    SET staattnum = 3
  WHERE starelid = 'clean_test'::regclass
    AND staattnum = 2;
@@ -1088,18 +1116,18 @@ SELECT count(*) FROM pg_statistic
  WHERE starelid = 'clean_test'::regclass;
 SELECT count(*) FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass;
-SELECT dbms_stats.clean_up_stats();
+SELECT dbms_stats.clean_up_stats() ORDER BY 1;
 SELECT count(*) FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass;
 -- No.18-1-8
-DELETE FROM dbms_stats._column_stats_locked
+DELETE FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass
    AND staattnum = 3;
 SELECT count(*) FROM pg_statistic
  WHERE starelid = 'clean_test'::regclass;
 SELECT count(*) FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass;
-SELECT dbms_stats.clean_up_stats();
+SELECT dbms_stats.clean_up_stats() ORDER BY 1;
 SELECT count(*) FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass;
 -- No.18-1-9
@@ -1108,33 +1136,122 @@ SELECT dbms_stats.lock_table_stats('clean_test');
 ALTER TABLE clean_test DROP COLUMN num;
 SELECT count(*) FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass;
-SELECT dbms_stats.clean_up_stats();
+SELECT dbms_stats.clean_up_stats() ORDER BY 1;
 SELECT count(*) FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass;
 -- No.18-1-10
-DELETE FROM dbms_stats._column_stats_locked
+DELETE FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass
    AND staattnum = 3;
 SELECT count(*) FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass;
-SELECT dbms_stats.clean_up_stats();
+SELECT dbms_stats.clean_up_stats() ORDER BY 1;
 SELECT count(*) FROM dbms_stats.column_stats_locked
  WHERE starelid = 'clean_test'::regclass;
-DELETE FROM dbms_stats._relation_stats_locked;
+DELETE FROM dbms_stats.relation_stats_locked;
 DROP TABLE clean_test;
 
 /*
  * No.19-1 dummy statistics view for general users privileges.
  */
-\c - regular_user
+SET SESSION AUTHORIZATION regular_user;
 -- No.19-1-1
 SELECT count(*) FROM dbms_stats.relation_stats_locked WHERE false;
 -- No.19-1-2
 SELECT count(*) FROM dbms_stats.column_stats_locked WHERE false;
 -- No.19-1-3
 SELECT count(*) FROM dbms_stats.stats WHERE false;
--- No.19-1-4
-SELECT count(*) FROM dbms_stats._relation_stats_locked WHERE false;
--- No.19-1-5
-SELECT count(*) FROM dbms_stats._columnns_user WHERE false;
+RESET SESSION AUTHORIZATION;
+
+/*
+ * No.20-1 confirm change at 1.3.5.
+ */
+SELECT CURRENT_USER;
+CREATE TABLE s0.st4 (a int, b text);
+CREATE INDEX i_st4_a on s0.st4 (a);
+CREATE VIEW s0.vst4 AS select * FROM s0.st4;
+GRANT SELECT ON s0.vst4 TO regular_user;
+INSERT INTO s0.st4 (SELECT a, a::text FROM generate_series(0, 999) a);
+ANALYZE s0.st4;
+SELECT dbms_stats.lock('s0.st4');
+DELETE FROM s0.st4;
+INSERT INTO s0.st4 (SELECT 1, a::text FROM generate_series(0, 999) a);
+ANALYZE s0.st4;
+
+EXPLAIN (COSTS OFF) SELECT * FROM s0.vst4 WHERE a = 1;
+EXPLAIN (COSTS OFF) SELECT * FROM s0.st4  WHERE a = 1;
+
+SET SESSION AUTHORIZATION regular_user;
+
+EXPLAIN (COSTS OFF) SELECT * FROM s0.st4  WHERE a = 1;
+EXPLAIN (COSTS OFF) SELECT * FROM s0.vst4 WHERE a = 1;
+
+SET pg_dbms_stats.use_locked_stats TO off;
+EXPLAIN (COSTS OFF) SELECT * FROM s0.vst4 WHERE a = 1;
 \c - super_user
+ALTER TABLE dbms_stats.relation_stats_locked OWNER TO super_user;
+
+SELECT dbms_stats.unlock('s0.st4');
+DROP TABLE s0.st4 CASCADE;
+
+/*
+ * No.21 anyarray stuff
+ */
+CREATE TABLE st_ary (i int, f float, d timestamp without time zone);
+INSERT INTO st_ary
+ (SELECT a, random(), '2016-3-25 00:00:00'::date + (a || 'day')::interval
+  FROM generate_series(0, 9999) a);
+ANALYZE st_ary;
+SELECT dbms_stats.lock('st_ary');
+
+/* Identifying the base type of the target anyarray */
+SELECT staattnum, dbms_stats.anyarray_basetype(stavalues1)
+FROM dbms_stats.column_stats_locked
+WHERE starelid = 'st_ary'::regclass
+ORDER BY staattnum;
+
+/* Generating subsidiary functions and casts */
+SELECT staattnum,
+	    dbms_stats.prepare_statstweak(
+			dbms_stats.anyarray_basetype(stavalues1)::regtype)
+FROM dbms_stats.column_stats_locked
+WHERE starelid = 'st_ary'::regclass
+ORDER BY staattnum;
+
+/* Tweaking stats */
+UPDATE dbms_stats.column_stats_locked
+SET stavalues1 = '{1,2,3,4,5}'::int[]
+WHERE starelid = 'st_ary'::regclass AND staattnum = 1;
+UPDATE dbms_stats.column_stats_locked
+SET stavalues1 = '{1.1,2.2,3.3,4.4,5.5}'::float8[]
+WHERE starelid = 'st_ary'::regclass AND staattnum = 2;
+UPDATE dbms_stats.column_stats_locked
+SET stavalues1 =
+  (SELECT ARRAY(SELECT '2016-1-1 0:0:0'::timestamp without time zone +
+                (i || 'day')::interval
+                FROM generate_series(0, 10) i))
+WHERE starelid = 'st_ary'::regclass AND staattnum = 3;
+
+SELECT staattnum, stavalues1 FROM dbms_stats.column_stats_locked
+WHERE starelid = 'st_ary'::regclass
+ORDER BY staattnum;
+
+/* Dropping tweak stuff */
+SELECT proname FROM pg_proc p JOIN pg_namespace n ON n.oid = p.pronamespace
+WHERE n.nspname = 'dbms_stats' AND p.proname LIKE '\_%\_anyarray'
+ORDER BY proname;
+
+SELECT staattnum,
+	    dbms_stats.drop_statstweak(
+			dbms_stats.anyarray_basetype(stavalues1)::regtype)
+FROM dbms_stats.column_stats_locked
+WHERE starelid = 'st_ary'::regclass
+ORDER BY staattnum;
+
+SELECT proname FROM pg_proc p JOIN pg_namespace n ON n.oid = p.pronamespace
+WHERE n.nspname = 'dbms_stats' AND p.proname LIKE '\_%\_anyarray'
+ORDER BY proname;
+
+/* Immediately unlock for safety */
+SELECT dbms_stats.unlock('st_ary');
+DROP TABLE st_ary;
